/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the ViewMyCurrency extension.
 *
 * The Initial Developer of the Original Code is
 * Will Moffat.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//dump("// $Id: WILL_ViewMyCurrency.js,v 1.1 2009/03/25 21:33:51 bogdan Exp $;\n");

VMC.ViewMyCurrency = function(gui) {
  var self=this;

  //public functions
  this.toggleHideOriginal           = toggleHideOriginal;
  this.getTargetCurrency            = getTargetCurrency;
  this.getNextTargetCurrency        = getNextTargetCurrency;
  this.changeToNextTargetCurrency   = changeToNextTargetCurrency;
  this.disableConversion            = disableConversion;
  this.updateBrowsers               = updateBrowsers;
  this.die                          = die;
  this.getRateFunc                  = null; // pointer to exchangeRateCache.getRate //WILL: make private but needed by VMC.PopupConvertToolbarManager ?
  this.observe                      = observe;
  this.updateGuiCallback            = function() { VMC.log('ERR','null callback - ViewMyCurrencyGui did not initialise'); }; 
  this.prefChangedCallBack          = prefChangedCallBack;

  // public vars
  this.gui           = gui; //WILL: TODO: I can probably break this link
  this.prefManager   = null;
  this.exchangeRateCache = null;
  this.tooltipMode = false;
  this.conversionProfileTable = null;

  // private vars:
  this._targetCurrency     = null;
  this._targetCurrencyList = null;
  this._sourceCurrencyList = null;
  this._delayToNextWebUpdate = 30*1000; // normally we run update after 30 sec. This gives user time to start using Firefox without delay added by me. This is only used for the initial web update. (Currency the only other webupdates that run are forced by the user in Options dialog)

  //private funcs
  this._onDOMContentLoaded           = _onDOMContentLoaded;
  //HACK:  this._initFromPrefs                = _initFromPrefs;
  this._setTargetCurrency            = _setTargetCurrency;
  this._scheduleInitialWebUpdate     = _scheduleInitialWebUpdate;
  this._delayed_initialWebUpdate     = _delayed_initialWebUpdate;
  this._delayed_kickStartGui         = _delayed_kickStartGui;
  this._scheduleCall                 = _scheduleCall;
  this._processFrame                 = _processFrame;
  this._initBeforeInstall            = _initBeforeInstall;
  this._initAfterInstall             = _initAfterInstall;
  this._updateProfile                = _updateProfile;

  this._initBeforeInstall();

  function _initBeforeInstall() {
    
    this.prefManager   = new VMC.PrefManager().init(); // Two-stage init required for Firefox 1.0
    this.prefChangedCallBack(VMC.PrefManager.keys.DEBUG_LEVEL); //initalise debug level by faking a pref change

    this._targetCurrencyList = new VMC.CurrencyList(); 
    this._sourceCurrencyList = new VMC.CurrencyList(); 

    var exCache = new VMC.ExchangeRateCache(this.prefManager.readRate, this.prefManager.writeRate,
					     this._sourceCurrencyList,this._targetCurrencyList);
    this.exchangeRateCache = exCache;
    this.getRateFunc = function(from,to) { return exCache.getRate(from,to); }; //WILL: is this really cleaner than just exposing the whole cache object

    this._scheduleCall('_delayed_kickStartGui',1); // we need to let Firefox GUI appear before doing our stuff. Prevents 'unreponsive script' warning.
  }

  function _delayed_kickStartGui() {
    this._delayedCall = null;
    this.gui.registerViewMyCurrency(this); //this will take care of running the Startup Wizard if required
  }

  //after delay milliseconds, calls this.funcName(null) 
  //WILL: TODO: why can't I just used setTimeout?
  function _scheduleCall(funcName,delay) {
    var info='Scheduling '+funcName+' in '+delay/1000+'s';
    VMC.log('INFO',info);
    if (this._delayedCall) { VMC.log('ERR','There is already a delayed call running!'); }
    this._delayedCall = new VMC.CCallWrapper(this, delay, funcName); 
    VMC.CCallWrapper.asyncExecute(this._delayedCall);
  }



    
  function _initAfterInstall(firstEverRun) {

    //HACK: don't I get this for free?  this._initFromPrefs(); //fill exchangeRateCache and this._targetCurrency from prefs


    var keys=VMC.PrefManager.keys;

    var interestingKeys = [keys.DEBUG_LEVEL,
			   // WILL: target list must be updated before source because we trigger updates using targetCurrencyList. 
			   keys.TARGET_CURRENCY_LIST, keys.SOURCE_CURRENCY_LIST, 
			   keys.HIDE_ORIGINAL_CURRENCY, 
			   keys.DECIMAL_SEPARATOR, keys.THOUSANDS_SEPARATOR, keys.FORCE_SPACE, keys.FORCE_SYMBOL_POS, keys.HIGHLIGHT_CHANGES,
			   keys.TOOLTIP_MODE,keys.LAYOUT_ORDER, keys.TARGET_CURRENCY_CODE, keys.RATE ];

    this.prefManager.registerWatcher(this,interestingKeys,'prefChangedCallBack');
  
    this.exchangeRateCache.webUpdater.addObserver(this);
  
    // activate initalise settings that have been read from prefs by faking pref changes
    for (var i=0; i<interestingKeys.length; i++) {
      var key=interestingKeys[i];
      if (key==keys.RATE) { continue; } // RATE is a branch, not an individual pref. The rates will be updated when the currency lists changes
      this.prefChangedCallBack(key, true); // skipUpdateBrowser = true     
    }

    if (firstEverRun) {
      this._delayed_initialWebUpdate(); //run the update immediately
    } else {
      this._scheduleInitialWebUpdate(); // run after a short delay - improves Firefox response and copes with failures
    }

    this.updateBrowsers(); // after initalising from the pregs, finally refresh the page
    this.updateGuiCallback(); // and the VMC GUI
  }



  function _scheduleInitialWebUpdate(immediately) {
    var delay;
    if (immediately) {
      delay=500; // well ok, not immediately, but after half a second. Gives this a chance to be cancelled by following pref updates
      if (this._delayedCall) { 
	this._delayedCall.cancel();
	this._delayedCall=null;
	VMC.log('INFO','(Had to cancel pending call)');
      }
    } else {
      delay=this._delayToNextWebUpdate;
      this._delayToNextWebUpdate *= 2; // double the back-off delay
    }
    this._scheduleCall('_delayed_initialWebUpdate',delay);
  }

  function _delayed_initialWebUpdate() {
    VMC.log('DBG','Doing delayed web update');
    this._delayedCall=null; // delete the call wrapper
    this.exchangeRateCache.updateFromWeb(false,true); //try to update any old or missing rates. forceUpdate=false, topOnFirstFailure=true
    //WILL: TODO: updates are now only on application startup - this should be periodic check
  }

  function observe(status) {
    //WILL: TODO: should I consider resetting update_interval if we get a END_OK? It doesn't matter for now, because we never update again.
    switch(status) {
    case 'END_OK' : this.updateBrowsers();  break; // the exchange rates have been successfully updated, now we update the current web pages.
    case 'END_ERRORS' : this._scheduleInitialWebUpdate(); break; // try updating again after a delay.
    default : // ignore all the rest (GUI takes care of updating gui)
    }
  } 

  //function newListFromCodes(codes) { //HACK: only required for hack below
  //var completeList = VMC.currencyDB.getCompleteList();
  //var currencyList = completeList.createListFromCodes(codes);
  //return currencyList;
  //}

  //HACK: this needs to be better thought out, right now it only has one entry in the conversionProfileTable 
  // and it's trashed each time the source/target lists changes
  function _updateProfile() {
    //HACK: hard code the table    //var sourceList = newListFromCodes(['USD','EUR','CAD','GBP']);
    var symbolTable = VMC.currencyDB.generateDefaultSymbolTableFor(this._sourceCurrencyList);
    var conversionProfile = new VMC.ConversionProfile(symbolTable);
    var defaultEntry = new VMC.ConversionProfileTableEntry('Default',conversionProfile, null );
    var table = new VMC.ConversionProfileTable( [defaultEntry] );
    this.conversionProfileTable = table;
  }

  function prefChangedCallBack(prefName,skipUpdateBrowser) {
    VMC.log('DBG','pref changed: '+prefName);

    var list=false; // did a currency list change?

    var keys=VMC.PrefManager.keys;

    switch(prefName) {
      // thse do not need a page refresh
    case keys.HIDE_IN_STATUSBAR       : return; // The GUI should respond to this
    case keys.DEBUG_LEVEL             : VMC.debugLevel=this.prefManager.getDebugLevel(); return; 
    case keys.TOOLTIP_MODE            : this.tooltipMode = this.prefManager.getTooltipMode(); return;

      // these require special list processing
    case keys.SOURCE_CURRENCY_LIST    : this.prefManager.readSourceCurrencyList(this._sourceCurrencyList); this._updateProfile(); list = true; break;
    case keys.TARGET_CURRENCY_LIST    : this.prefManager.readTargetCurrencyList(this._targetCurrencyList); list = true; break;

      // the page will be refreshed
    case keys.HIDE_ORIGINAL_CURRENCY  : this.hideOriginalCurrency  = this.prefManager.getHideOriginalCurrency(); break;
    case keys.DECIMAL_SEPARATOR       : VMC.Number.prototype.decimalSeparator   = this.prefManager.getDecimalSeparator(); break;
    case keys.THOUSANDS_SEPARATOR     : VMC.Number.prototype.thousandsSeparator = this.prefManager.getThousandsSeparator(); break;
    case keys.FORCE_SPACE             : VMC.NumberConversion.prototype.forceSpaceBetweenNumberAndSymbol = this.prefManager.getForceSpace(); break;
    case keys.LAYOUT_ORDER            : VMC.NumberConversion.prototype.targetInBrackets = this.prefManager.getLayoutOrder(); break;
    case keys.FORCE_SYMBOL_POS        : VMC.NumberConversion.prototype.forceSymbolPosition = this.prefManager.getForceSymbolPos(); break;
    case keys.HIGHLIGHT_CHANGES       : VMC.HtmlConversion.highlightChanges = this.prefManager.getHighlightChanges(); break; //WILL: TODO: need to force an update of nodes to see the effect. Could disableConversion and then re-enable
    case keys.TARGET_CURRENCY_CODE    : this._targetCurrency = this._targetCurrencyList.findFromCode(this.prefManager.getTargetCurrencyCode()); break;

    default : 
      if (prefName.match(keys.RATE)) {
	var allRatesSet = this.exchangeRateCache.setAllRatesFromPrefs(); //WILL: TODO: should I modify setAllRates to return false also if rates are old?
      }
      else { VMC.log('ERR','Saw unexpected pref '+prefName+' change'); return; }
    }
    
    if (list) { // did a list change?
      this.exchangeRateCache.sourceOrTargetListChanged();
      if (!skipUpdateBrowser) { //we delay initial webupdate
	this._scheduleInitialWebUpdate(true); //do it now!
      }

     //WILL: we can't set the target currency (which redoes the page) until we've updated the rates. 
     //update the target currency - if it's not on the list anymore, go back to null.
      this._setTargetCurrency(this._targetCurrencyList.findFromCode(this.prefManager.getTargetCurrencyCode())); //WILL: TODO: check this!
      // ( _setTargetCurrency calls updateBrowsers )
    } else {

      if (!skipUpdateBrowser) { // only need to redraw pages if we are currency converting
	if (this._targetCurrency || prefName==keys.TARGET_CURRENCY_CODE) {
	  // refresh page if we are currently converting. Always refresh if target changed (restore page if target=null)
	  this.updateGuiCallback();  // gui must change if targetCurrency or hideOriginal changed. FUTURE: also if there were errors to show.
	  this.updateBrowsers(); // we make the gui change first and then do the work. It looks more responsive

	}
      }
    }

  }


  //////////////
  function die() {
    //////////////
    // we got an unload event

    if (VMC.CCallWrapper.mPendingCalls.length) {
      this._delayedCall.cancel();
    } else {
      this.exchangeRateCache.webUpdater.removeObserver(this);
    }

    this.prefManager.unregisterWatcher(this); //kills all watchers. Included that of GUI
  }
//   /////////////////////
//   function _initFromPrefs() { HACK: TO DELETE
//     ///////////////////
    
//     // quick sanity check
//     var missingPrefs=this.prefManager.missingRequiredPrefs();
//     if (missingPrefs) { VMC.alert(  'The following preferences are missing:\n'+missingPrefs+  '\n\nThe startup check should have prevented this'); }

//     this.prefManager.readSourceCurrencyList(this._sourceCurrencyList);
//     this.prefManager.readTargetCurrencyList(this._targetCurrencyList);

//     VMC.CurrencyList.prototype.isValid(this._sourceCurrencyList);
//     VMC.CurrencyList.prototype.isValid(this._targetCurrencyList);

//     this.exchangeRateCache.sourceOrTargetListChanged();

//     //WILL: do I need the complete list somewhere?
//     this._targetCurrency             = this.prefManager.getTargetCurrency(); //null is allowed --> disabled

//     //WILL: if I change a list, don't I need to update the cache?
//   }

  function toggleHideOriginal() {
    this.hideOriginalCurrency = ! this.hideOriginalCurrency;
    this.prefManager.setHideOriginalCurrency(this.hideOriginalCurrency);
    //_watchHideOriginal will take care of updating browsers
  }

  function getTargetCurrency()     {  return this._targetCurrency;  }
  function getNextTargetCurrency() {  return this._targetCurrencyList.getNext(this._targetCurrency);  }
  function changeToNextTargetCurrency() { this._setTargetCurrency(this.getNextTargetCurrency());     }
  function disableConversion()          { this._setTargetCurrency(null);                             }

  //update the target currency - if it's not on the list go to disabled
  function _setTargetCurrency(newTargetCurrency) {
    var newTarget=null;
    if (newTargetCurrency) {
      if (this._targetCurrencyList.existsInList(newTargetCurrency)) { newTarget=newTargetCurrency; }
      else { VMC.log('INFO','Setting targetCurrency to null as '+newTargetCurrency+' not in '+this._targetCurrencyList); }
    }
    if (this._targetCurrency != newTarget) {
      if (!newTarget) { this.updateGuiCallback("OFF")   } // give instant visual cue that we are going OFF (prefs can take .5 sec to update?)
      // we are changing currency
      this.prefManager.setTargetCurrencyCode( newTarget ? newTarget.code : null);
      //rely on the pref change to do this.updateGuiCallback(); this._targetCurrency=newTarget; to this.updateBrowsers();
    }
  }

  //WILL: TODO: there is some duplication of code between updateBrowsers and _onDOMContentLoaded
  function updateBrowsers() {
    if (this.tooltipMode) {return; }

    if (this._targetCurrency) { this.updateGuiCallback("START") } // show visual cue that we are starting to convert.
    this._processFrame(top);
  }

  function _processFrame(f) {
    if (!f) {return; }
    VMC.log('DBG','Processing frame '+f+' location='+f.location);
    
    if (f != top) { // don't modify the browser itself!
      if (this._targetCurrency) {
	//force active pages to update since we just changed the target //WILL: CHECK is this force
	VMC.HtmlConversion.changePageToTargetCurrencyUsingNodes(f.document, this._targetCurrency, this.hideOriginalCurrency, this.conversionProfileTable, this.getRateFunc, this.updateGuiCallback); 
      } else {
	VMC.HtmlConversion.restorePage(f.document);
      }
    }
    
    for (var i=0;i<f.frames.length; i++) {
      this._processFrame(f.frames[i]);
    }
  }


  // happens every time I load a page

  ////////////////////////
  function _onDOMContentLoaded(e) {
    ////////////////////////
    if (this.tooltipMode) { return; }

    var startTime=new Date();

    var document = e.originalTarget;

    if (VMC.HtmlConversion.isURLtoAvoid(document,'Loaded page')) { // WILL: CHECK:  we check again in changePageTo... 
      this.updateGuiCallback('AVOID');
    } else {
      if (this._targetCurrency) {
	this.updateGuiCallback("START"); // show visual cue that we are starting
	VMC.HtmlConversion.changePageToTargetCurrencyUsingNodes(document, this._targetCurrency, this.hideOriginalCurrency,
								this.conversionProfileTable, this.getRateFunc, this.updateGuiCallback);
      }
    }

    var endTime=new Date();
    VMC.log('INFO','Extra delay due to me was '+ (endTime-startTime) + 'ms');
  }

};
